--[[
# Copyright 2020 Quarq
# This file is part of A-Bomb: Azeroth Bombardier.
# A-Bomb: Azeroth Bombardier is distributed under a BSD License.
# It is provided AS-IS and all warranties, express or implied, including, but not
# limited to, the implied warranties of merchantability or fitness for a particular
# purpose, are disclaimed.  See the LICENSE file for full information.
--]]
local MYNAME, _ = ...
MYNAME = GetAddOnMetadata(MYNAME, "X-ShortName") or MYNAME
local Addon = _G[MYNAME]
assert(Addon)

local sprintf = string.format
local floor = math.floor


local MSF = {}


local FONTSIZE = 12

local SUCCESS_SOUND = 6674
local FAIL_SOUND = 6675
local SOUND_DELAY = 0.50	-- seconds


local function dot(colorname, size)
	size = size or FONTSIZE-2

	local color = Addon.COLORS_BY_NAME[colorname]
	local r,g,b = Addon.hsv(color.hue, 1, 1)
	r,g,b = r*255, g*255, b*255

	-- |Tpath:height[:width[:offsetX:offsetY:[textureWidth:textureHeight:leftTexel:rightTexel:topTexel:bottomTexel[:rVertexColor:gVertexColor:bVertexColor]]]]|t
	local s = "|T"
	s = s .. "Interface/Addons/A-Bomb/textures/target-outlined"
	s = s .. ":"..size..":"..size		-- :height:width
	s = s .. ":0:0"					-- :offsetX:offsetY
	s = s .. ":32:32:0:32:0:32"		-- :textureWidth:textureHeight:leftTexel:rightTexel:topTexel:bottomTexel
	s = s .. ":"..r..":"..g..":"..b	-- :rVertexColor:gVertexColor:bVertexColor
	s = s .. "|t"

	return s
end

local function icon_string(icon, size)
	size = size or (FONTSIZE+2)

	-- |Tpath:height[:width[:offsetX:offsetY:[textureWidth:textureHeight:leftTexel:rightTexel:topTexel:bottomTexel[:rVertexColor:gVertexColor:bVertexColor]]]]|t
	local s = "|T"
	s = s .. "Interface/Addons/A-Bomb/textures/"..icon
	s = s .. ":"..size..":"..size		-- :height:width
	s = s .. ":0:0"					-- :offsetX:offsetY
	s = s .. ":16:16:0:16:0:16"		-- :textureWidth:textureHeight:leftTexel:rightTexel:topTexel:bottomTexel
	-- s = s .. ":"..r..":"..g..":"..b	-- :rVertexColor:gVertexColor:bVertexColor
	s = s .. "|t"

	return s
end


function Addon : NewMissionStatus(mission)
	local msf = Mixin(CreateFrame("FRAME"), MSF)
	msf:SetMission(mission)
	return msf
end


function MSF : SetMission(mission)

	self.mission = mission

	self.bombs = 0
	self.totalkills = 0
	self.survivors = 0
	self.kills = {}

	if (mission == nil) then
		return
	end

	local dx = 0
	local dy = 0


	if (mission.kills) then
		self.kills_label = self:new_label()
		self.kills_label:SetPoint("TOPRIGHT", self, "TOPRIGHT", dx, dy)
		self.kills_label:Show()
		dx = dx - 110
	end

	if (mission.bombs) then
		self.bomb_label = self:new_label()
		self.bomb_label:SetPoint("TOPRIGHT", self, "TOPRIGHT", dx, dy)
		self.bomb_label:Show()
		dx = dx - 122
	end

	if (mission.survivors) then
		self.survivors_label = self:new_label()
		self.survivors_label:SetPoint("TOPRIGHT", self, "TOPRIGHT", dx, dy)
		self.survivors_label:Show()
		dx = dx - 80
	end

	if (mission.exclude) then
		self.exclude_label = self:new_label()
		self.exclude_label:SetPoint("TOPRIGHT", self, "TOPRIGHT", dx, dy)
		self.exclude_label:Show()
		dx = dx - 80
	end

	self:SetSize(math.abs(dx), FONTSIZE)
	self:SetPoint("TOPRIGHT", WorldMapFrame.ScrollContainer, "TOPRIGHT", -8, -1)

	-- assign initial values with 0 progress counters
	self:UpdateProgress()
end


function MSF : UpdateProgress()
	if (self.mission == nil) then
		return
	end

	local mission = self.mission
	local status = self	-- bombs, totalkills, kills[color], survivors are my properties

	if (mission.kills) then
		local label = self.kills_label

		local icon
		if (mission.color) then
			icon = ((status.kills[mission.color] or 0) >= mission.kills) and "succeeded" or "box"
			label:SetText(sprintf("%s Kills: %d/%d %s", icon_string(icon), (status.kills[mission.color] or 0), mission.kills, dot(mission.color)))
		else
			icon = ((status.totalkills or 0) >= mission.kills) and "succeeded" or "box"
			label:SetText(sprintf("%s Kills: %d/%d", icon_string(icon), (status.totalkills or 0), mission.kills))
		end

		if (icon == "succeeded" and not label.alerted) then
			label.alerted = true
			C_Timer.After(SOUND_DELAY, function() PlaySound(SUCCESS_SOUND) end)
		end
	end

	if (mission.bombs) then
		local label = self.bomb_label

		local icon = ((status.bombs or 0) <= mission.bombs) and "box" or "failed"
		label:SetText(sprintf("%s Bombs: %d/%d", icon_string(icon), (status.bombs or 0), mission.bombs))

		if (icon == "failed" and not label.alerted) then
			label.alerted = true
			C_Timer.After(SOUND_DELAY, function() PlaySound(FAIL_SOUND) end)
		end
	end

	if (mission.survivors) then
		local label = self.survivors_label

		if (mission.color) then
			label:SetText(sprintf("%s %.0f%% %s survivors", icon_string("box"), mission.survivors*100, dot(mission.color)))
		else
			label:SetText(sprintf("%s %.0f%% survivors", icon_string("box"), mission.survivors*100))
		end
	end

	if (mission.exclude) then
		local label = self.exclude_label

		local icon = ((status.kills[mission.exclude] or 0) == 0) and "box" or "failed"
		label:SetText(sprintf("%s Avoid %s", icon_string(icon), dot(mission.exclude)))

		if (icon == "failed" and not label.alerted) then
			label.alerted = true
			C_Timer.After(SOUND_DELAY, function() PlaySound(FAIL_SOUND) end)
		end
	end
end


function MSF : new_label()
	local label = self:CreateFontString(nil, "ARTWORK")
	label:SetFont("Fonts/FRIZQT__.TTF", FONTSIZE)
	label:SetTextColor(1,1,1, 1)
	label:SetShadowOffset(1,-1)
	label:SetShadowColor(0,0,0, 1)
	return label
end
